@@ -713,148 +713,3 @@ BOOL cmd_condition_re(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
     return TRUE;
 }
 
-#if defined(HAVE_MIGEMO_H)
-#include <migemo.h>
-
-static migemo* gMigemo;
-static regex_t* gReg;
-
-void migemo_init()
-{
-    char buf[PATH_MAX];
-    char migemodir[PATH_MAX];
-    gMigemo = migemo_open(NULL);
-
-    snprintf(migemodir, PATH_MAX, "%s", SYSTEM_MIGEMODIR);
-
-    snprintf(buf, PATH_MAX, "%s/utf-8/migemo-dict", migemodir);
-    if(migemo_load(gMigemo, MIGEMO_DICTID_MIGEMO, buf) == MIGEMO_DICTID_INVALID) {
-        fprintf(stderr, "%s is not found\n", buf);
-        exit(1);
-    }
-    snprintf(buf, PATH_MAX, "%s/utf-8/roma2hira.dat", migemodir);
-    if(migemo_load(gMigemo, MIGEMO_DICTID_ROMA2HIRA, buf) == MIGEMO_DICTID_INVALID) {
-        fprintf(stderr, "%s is not found\n", buf);
-        exit(1);
-    }
-    snprintf(buf, PATH_MAX, "%s/utf-8/hira2kata.dat", migemodir);
-    if(migemo_load(gMigemo, MIGEMO_DICTID_HIRA2KATA, buf) == MIGEMO_DICTID_INVALID) {
-        fprintf(stderr, "%s is not found\n", buf);
-        exit(1);
-    }
-    snprintf(buf, PATH_MAX, "%s/utf-8/han2zen.dat", migemodir);
-    if(migemo_load(gMigemo, MIGEMO_DICTID_HAN2ZEN, buf) == MIGEMO_DICTID_INVALID) {
-        fprintf(stderr, "%s is not found\n", buf);
-        exit(1);
-    }
-}
-
-void migemo_final()
-{
-    onig_end();
-    migemo_close(gMigemo);
-    if(gReg) onig_free(gReg);
-    gReg = NULL;
-}
-
-BOOL cmd_migemo_match(sObject* nextin, sObject* nextout, sRunInfo* runinfo)
-{
-    BOOL quiet = sRunInfo_option(runinfo, "-quiet");
-
-    if(runinfo->mFilter && runinfo->mArgsNumRuntime == 2) {
-        runinfo->mRCode = RCODE_NFUN_FALSE;
-        char* target = SFD(nextin).mBuf;
-        char* regex = runinfo->mArgsRuntime[1];
-
-        if(regex[0] == 0) {
-            runinfo->mRCode = 0;
-            if(!quiet) {
-                char buf[1024];
-                int n = snprintf(buf, 1024, "0%d\n", (int)strlen(target));
-                if(!fd_write(nextout, buf, n)) {
-                    err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                    runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                    return FALSE;
-                }
-            }
-        }
-        else {
-            OnigUChar * p = migemo_query(gMigemo, regex);
-            if(p == NULL) {
-                err_msg("migemo query failed", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                migemo_release(gMigemo, (unsigned char*) p);
-                return FALSE;
-            }
-
-            /// modify query ///
-            char* p2 = MALLOC(strlen(p)*2 + 1);
-
-            char* _p = p;
-            char* _p2 = p2;
-
-            while(*_p) {
-                if(*_p == '+') {
-                    *_p2++ = '\\';
-                    *_p2++ = *_p++;
-                }
-                else {
-                    *_p2++ = *_p++;
-                }
-            }
-            *_p2 = 0;
-
-            /// make regex ///
-            regex_t* reg;
-            OnigErrorInfo err_info;
-
-            int r = onig_new(&reg, p2, p2 + strlen(p2), ONIG_OPTION_DEFAULT, ONIG_ENCODING_UTF8, ONIG_SYNTAX_DEFAULT,  &err_info);
-
-            FREE(p2);
-            migemo_release(gMigemo, (unsigned char*) p);
-
-            if(r != ONIG_NORMAL && r != 0) {
-                err_msg("regex of migemo query failed", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                onig_free(reg);
-                return FALSE;
-            }
-
-            OnigRegion* region = onig_region_new();
-            int r2 = onig_search(reg, target, target + strlen(target), target, target + strlen(target), region, ONIG_OPTION_NONE);
-
-            if(r2 == 0) {
-                runinfo->mRCode = 0;
-                if(!quiet) {
-                    char buf[1024];
-                    int n = snprintf(buf, 1024, "%d\n%d\n", region->beg[0], region->end[0]);
-                    if(!fd_write(nextout, buf, n)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                        onig_region_free(region, 1);
-                        onig_free(reg);
-                        return FALSE;
-                    }
-                }
-            }
-            else {
-                if(!quiet) {
-                    char buf[1024];
-                    int n = snprintf(buf, 1024, "-1\n-1\n");
-                    if(!fd_write(nextout, buf, n)) {
-                        err_msg("signal interrupt", runinfo->mSName, runinfo->mSLine, runinfo->mArgs[0]);
-                        runinfo->mRCode = RCODE_SIGNAL_INTERRUPT;
-                        onig_region_free(region, 1);
-                        onig_free(reg);
-                        return FALSE;
-                    }
-                }
-            }
-
-            onig_region_free(region, 1);
-            onig_free(reg);
-        }
-    }
-
-    return TRUE;
-}
-
-#endif
