@@ -498,12 +498,9 @@ static int readline_bind_cr(int count, int key)
 
     sObject* block = BLOCK_NEW_STACK();
 
-    sObject* cmdline = STRING_NEW_STACK("");
-    string_push_back3(cmdline, rl_line_buffer, rl_point);
-
     int sline = 1;
     sObject* current_object = gCurrentObject;
-    BOOL result = parse(string_c_str(cmdline), "readline", &sline, block, &current_object);
+    BOOL result = parse(rl_line_buffer, "readline", &sline, block, &current_object);
 
     /// in the block? get the block
     if(!result && (SBLOCK(block).mCompletionFlags & (COMPLETION_FLAGS_BLOCK|COMPLETION_FLAGS_ENV_BLOCK))) {
